<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/26/16
 * Time: 11:25 AM
 */

namespace qfq;

use qfq;
use qfq\Store;
use qfq\UserException;

require_once(__DIR__ . '/../../qfq/store/Store.php');
require_once(__DIR__ . '/../../qfq/Constants.php');
require_once(__DIR__ . '/../../qfq/helper/KeyValueStringParser.php');
require_once(__DIR__ . '/../../qfq/exceptions/UserException.php');

class HelperFormElement
{

    /**
     * Take all rows from field $element['parameter'] and merge them with $element. '$element' grows in size.
     *
     * Iterates over all FormElements in $elements.
     * If $element['parameter'] exist and is filled: explode the content and append them to the current array.
     *
     * @param array $elements
     * @throws UserException
     */
    public static function explodeParameter(array &$element)
    {
        // Something to explode?
        if (isset($element['parameter']) && $element['parameter'] !== '') {
            // Explode
            $arr = KeyValueStringParser::parse($element['parameter'], "=", "\n");
            // Check if some of the exploded keys conflict with existing keys
            $checkKeys = array_keys($arr);
            foreach ($checkKeys AS $checkKey) {
                if (isset($element[$checkKey])) {
                    $store = Store::getInstance();
                    $store->setVar(SYSTEM_FORM_ELEMENT, $element['name'] . ' / ' . $element['id'], STORE_SYSTEM);
                    $store->setVar(SYSTEM_FORM_ELEMENT_COLUMN, 'parameter', STORE_SYSTEM);
                    throw new UserException("Found reserved keyname '$checkKey'", ERROR_RESERVED_KEY_NAME);
                }
            }
            $element = array_merge($element, $arr);
        }
    }

    /**
     * @param array $elements
     * @throws \qfq\UserException
     */
    public static function explodeParameterInArrayElements(array &$elements)
    {
        foreach ($elements AS $key => $element) {
            self::explodeParameter($element);
            $elements[$key] = $element;
        }
    }

    /**
     * @param $field
     * @param $id
     * @return string
     */
    public static function buildFormElementId($field, $id)
    {
        return ($field . ':' . $id);
    }

}