<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/2/16
 * Time: 10:57 PM
 */

namespace qfq;

use qfq;
use qfq\CodeException;

require_once(__DIR__ . '/../../qfq/Constants.php');

/**
 * Class Sanitize
 * @package qfq
 */
class Sanitize {

    private function __construct() {
        // Class should never be instantiated
    }

    /**
     * Sanitize: check value by sanitize class. Return verified value or empty string if failed.
     *
     * @param string $value - value to check
     * @param string $sanitizeClass - class used to check
     * @return string - if check failed: empty string, if check is ok: value
     * @throws CodeException
     */
    public static function sanitize($value, $sanitizeClass = SANITIZE_DEFAULT) {

        switch ($sanitizeClass) {
            case SANITIZE_DEFAULT:
            case SANITIZE_ALLOW_DIGIT:
                if (!is_numeric($value)) {
                    $value = '';
                }
                break;

            case SANITIZE_ALLOW_ALNUMX:
                // replace ALNUMX valid characters, which are not part of ctype_alnum, by valid ctype_alnum characters, to fake ctype_alnum
                // definetely forbidden: ' " \ % $
                if (!ctype_alnum(str_replace(array('@', '-', '_', '.', ',', ';', ' ', '/', '(', ')'), 'a', $value)))
                    $value = '';
                break;

            case SANITIZE_ALLOW_ALL:
                break;

            default:
                throw new CodeException("Sanitize class '$sanitizeClass' unknown. Used to sanitize GET/POST Variable.", ERROR_UNKNOW_SANITIZE_CLASS);
                break;
        }

        return ($value);
    }

}