<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/9/16
 * Time: 10:26 PM
 */

namespace qfq;

use qfq\CodeException;
use qfq\UserException;
use qfq\OnArray;
use qfq\KeyValueStringParser;

require_once(__DIR__ . '/../../qfq/helper/OnArray.php');
require_once(__DIR__ . '/../../qfq/Constants.php');
require_once(__DIR__ . '/../../qfq/exceptions/CodeException.php');

/**
 * Class Sip
 * @package qfq\store
 */
class Sip {

// $_SESSION['fe_user_uid'] = <fe_user_uid>
// $_SESSION[$sip] => <urlparam>  >> $_SESSION['badcaffee1234'] => 'form=Person&r=1&z=5678'
// $_SESSION[$urlparam] => <sip>  >> $_SESSION['form=Person&r=1&z=5678'] => 'badcaffee1234'

    private $phpUnit = false;
    private $staticUniqId = false;

    function __construct($sessionname, $phpUnit = false) {

        $this->phpUnit = $phpUnit;

        if ($sessionname == "") {
            throw new CodeException('Missing "sessionname"', ERROR_MISSING_SESSIONNAME);
        }

        session_name();

        if ($phpUnit) {
            $_SESSION = null;
            @session_start();
        } else {
            session_start();
        }

        // Typo3: remember logged in FE User
        if (isset($GLOBALS["TSFE"]->fe_user->user["uid"]) && !isset($_SESSION[SESSION_FE_USER_UID])) {
            $_SESSION[SESSION_FE_USER_UID] = $GLOBALS["TSFE"]->fe_user->user["uid"];
        }

    }

    /**
     * @param string $queryString Possible variants:
     *   * http://www.math.uzh.ch/index.php?a=1&s=4b3403665fea6&r=45&type=99&id=person
     *   * index.php?a=1&s=4b3403665fea6&r=45&type=99&id=person
     *   * ?a=1&s=4b3403665fea6&r=45&type=99&id=person
     *   * a=1&s=4b3403665fea6&r=45&type=99&id=person
     *
     * @param string $mode Possible values: RETURN_URL|RETURN_SIP
     * @return string
     *  * mode=RETURN_URL: return complete URL
     *  * mofr=RETURN_SIP: returns only the sip
     * @throws CodeException
     * @throws UserException
     */
    public function queryStringToSip($queryString, $mode = RETURN_URL, $scriptName = 'index.php') {

        // Validate: Check if still the same fe_user is logged in.
        if (isset($GLOBALS["TSFE"]->fe_user->user["uid"])) {
            if ($_SESSION[SESSION_FE_USER_UID] != $GLOBALS["TSFE"]->fe_user->user["uid"]) {
                unset($_SESSION);
            }
        }

        $clientArray = array();
        $sipArray = array();

        // Split URL parameter:
        $paramArray = KeyValueStringParser::parse($queryString, "=", "&");

        // Split parameter between Script, Client and SIP
        $script = $this->splitParamClientSip($paramArray, $clientArray, $sipArray);

        // sort array to guarantee identical respresentation in $_SESSION. Param 'a, r, b, ...' should be saved as 'a, b, r, ..'
        OnArray::sortKey($sipArray);

        // Generate keyname for $_SESSION[]
        $sipParamString = OnArray::toString($sipArray);

        if (isset($_SESSION[$sipParamString])) {
            $s = $_SESSION[$sipParamString];
        } else {
            // Not found: create new entry
            $s = $this->sipUniqId();
            $_SESSION[$sipParamString] = $s;
            $_SESSION[$s] = $sipParamString;
        }

        if ($mode == RETURN_URL) {
            // Append SIP to final parameter
            $clientArray[CLIENT_SIP] = $s;

            if ($script[0] === '?')
                $script = $scriptName . $script;

            return $script . OnArray::toString($clientArray);
        } else {
            return $s;
        }
    }

    /**
     * @param $param
     * @param $script
     * @param array $clientArray
     * @param array $sipArray
     * @throws CodeException
     * @throws UserException
     */
    private function splitParamClientSip($paramArray, array &$clientArray, array &$sipArray) {

        $script = '';

        // Possible variants:
        //   http://www.math.uzh.ch/index.php?a=1&s=4b3403665fea6&r=45&type=99&id=person
        //   index.php?a=1&s=4b3403665fea6&r=45&type=99&id=person
        //   ?a=1&s=4b3403665fea6&r=45&type=99&id=person
        //   a=1&s=4b3403665fea6&r=45&type=99&id=person
        $flagFirst = true;
        foreach ($paramArray AS $key => $value) {

            // first key/value pair: a) save potential existing url/script part, b) if no keyname is specified, call it 'id'.
            if ($flagFirst) {
                $flagFirst = false;

                $script = $this->splitAndFix($key, $value);
            }

            // copy every parameter either to $clientArray or to $sipArray
            switch ($key) {
                //special T3 parameter.
                case 'L':
                case 'type':
                case 'id':
                    $clientArray[$key] = $value;
                    break;
                case CLIENT_SIP:
                    throw new CodeException('SIP Parameter ist not allowed to be stored as a regular URL Parameter', ERROR_SIP_NOT_ALLOWED_AS_PARAM);
                default:
                    $sipArray[$key] = $value;
                    break;
            }
        }

        return $script;
    }

    /**
     * Fix first parameter mix of hostname / script / Get parameter and optional missing keyname
     *
     * @param $key
     * @param $value
     * @return string Part upto first '?',
     */
    private function splitAndFix(&$key, &$value) {

        $script = '';
        $tmpArray = explode('?', $key, 2);

        if (count($tmpArray) > 1) {
            $script = $tmpArray[0];
            $key = $tmpArray[1]; // new key, now without the hostname / scriptname.
        }
        $script .= '?';


        // empty value: keyname omitted - define keyname as 'id'
        if ($value === '') {
            $value = $key;
            $key = 'id';
        }

        return $script;
    }

    /**
     * @param bool|false $staticUniqId
     * @return bool|string
     */
    public function sipUniqId($staticUniqId = false) {
        if ($this->phpUnit) {
            if ($staticUniqId !== false) {
                $this->staticUniqId = $staticUniqId;
            }
            return $this->staticUniqId;
        }

        return uniqid();
    }

    /**
     * Retrieve Params stored in $_SESSION[$s]
     *
     * @param $s
     * @return array Parameter Array
     * @throws UserException
     * @throws \qfq\UserException
     */
    public function getVarsFromSip($s) {

        # Check if parameter is manipulated
        if (strlen($s) != 13) {
            throw new UserException("Broken Parameter", ERROR_BROKEN_PARAMETER);
        }

        # Check if still the same fe_user is logged in.
        if (isset($GLOBALS["TSFE"]->fe_user->user["uid"])) {
            if ($_SESSION[SESSION_FE_USER_UID] != $GLOBALS["TSFE"]->fe_user->user["uid"]) {
                throw new UserException("No user logged in or user changed - please reload the previous site.", ERROR_FE_USER_UID_CHANGED);
            }
        }

        # Check if index 's' exists.
        if (!isset($_SESSION[$s])) {
            throw new UserException("SIP '$s' not registered - please reload the previous site and try again.", ERROR_SIP_NOT_FOUND);
        }

        // Decode parameter
        return KeyValueStringParser::parse($_SESSION[$s], "=", "&");
    }

    /**
     * Returns the sip for the given querystring. The querystring has to be sorted.
     *
     * @param $queryString
     * @return mixed
     */
    public function getSipFromQueryString($queryString) {
        if (isset($_SESSION[$queryString])) {
            return $_SESSION[$queryString];
        }
        return false;
    }

    /**
     * Returns the querystring for the given $sip
     *
     * @param $sip
     * @return bool
     */
    public function getQueryStringFromSip($sip) {
        if (isset($_SESSION[$sip])) {
            return $_SESSION[$sip];
        }
        return false;
    }

}