<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/2/16
 * Time: 11:10 PM
 */

namespace qfq;

require_once(__DIR__ . '/../../qfq/helper/Sanitize.php');
require_once(__DIR__ . '/../../qfq/exceptions/CodeException.php');


class SanitizeTest extends \PHPUnit_Framework_TestCase {

    public function testSanitize() {

        # Violates SANITIZE class: SANITIZE string is always an empty string.
        # Access are cached: use new variables for every test.

        # Check '1'
        $this->assertEquals('1', Sanitize::sanitize('1', SANITIZE_ALLOW_DIGIT), "SANITIZE_DIGIT fails");
        $this->assertEquals('1', Sanitize::sanitize('1', SANITIZE_ALLOW_ALNUMX), "SANITIZE_ALNUMX fails");
        $this->assertEquals('1', Sanitize::sanitize('1', SANITIZE_ALLOW_ALL), "SANITIZE_ALL fails");

        # Check '-3'
        $this->assertEquals('-3', Sanitize::sanitize('-3', SANITIZE_ALLOW_DIGIT), "SANITIZE_DIGIT fails");
        $this->assertEquals('-3', Sanitize::sanitize('-3', SANITIZE_ALLOW_ALNUMX), "SANITIZE_ALNUMX fails");
        $this->assertEquals('-3', Sanitize::sanitize('-3', SANITIZE_ALLOW_ALL), "SANITIZE_ALL fails");

        # Check 'a'
        $this->assertEquals('', Sanitize::sanitize('a', SANITIZE_ALLOW_DIGIT), "SANITIZE_DIGIT fails");
        $this->assertEquals('a', Sanitize::sanitize('a', SANITIZE_ALLOW_ALNUMX), "SANITIZE_ALNUMX fails");
        $this->assertEquals('a', Sanitize::sanitize('a', SANITIZE_ALLOW_ALL), "SANITIZE_ALL fails");


        # Check 'a@-_.,;Z09'
        $this->assertEquals('', Sanitize::sanitize('a@-_.,;Z09', SANITIZE_ALLOW_DIGIT), "SANITIZE_DIGIT fails");
        $this->assertEquals('a@-_.,;Z09', Sanitize::sanitize('a@-_.,;Z09', SANITIZE_ALLOW_ALNUMX), "SANITIZE_ALNUMX fails");
        $this->assertEquals('a@-_.,;Z09', Sanitize::sanitize('a@-_.,;Z09', SANITIZE_ALLOW_ALL), "SANITIZE_ALL fails");

        # Check 'a+Z09'
        $this->assertEquals('', Sanitize::sanitize('a+Z09', SANITIZE_ALLOW_DIGIT), "SANITIZE_DIGIT fails");
        $this->assertEquals('', Sanitize::sanitize('a+Z09', SANITIZE_ALLOW_ALNUMX), "SANITIZE_ALNUMX fails");
        $this->assertEquals('a+Z09', Sanitize::sanitize('a+Z09', SANITIZE_ALLOW_ALL), "SANITIZE_ALL fails");
    }

    /**
     * @expectedException \qfq\CodeException
     *
     */
    public function testSanitizeException() {
        Sanitize::sanitize('Hello World', 'invalid sanitize class');
    }

}
