<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/2/16
 * Time: 9:16 PM
 */

namespace qfq;

use qfq;
use qfq\Store;

//use qfq\exceptions\CodeException;

require_once(__DIR__ . '/../../qfq/store/Store.php');
require_once(__DIR__ . '/../../qfq/Constants.php');
require_once(__DIR__ . '/../../qfq/exceptions/CodeException.php');


class StoreTest extends \PHPUnit_Framework_TestCase {

    /**
     * @var Store
     */
    private $store = null;
    private $setUpDone = false;

    public function setUp() {
        // Client Variables has to setup before the first instantiation of 'Store' 
        $_GET[CLIENT_RECORD_ID] = '1234';
//        $_GET[CLIENT_SIP] = '12badcaffee34';
        $_GET['key01'] = '1234';
        $_POST['key02'] = '2345';
        $_POST['key03'] = '3456';
        $_POST[CLIENT_FORM] = 'testformnameDoNotChange';
        $_GET['key04'] = '4567';
        $_POST['key04'] = '5678';

        $this->store = Store::getInstance("\n; some comment\n" . TYPO3_FORM . "=testformnameDoNotChange\n" . TYPO3_DEBUG_SAVE . "=6\n" . TYPO3_DEBUG_LOAD . "=7\n", true);

    }

    public function testGetInstance() {
        $a = Store::getInstance('', true);
        $b = Store::getInstance('', true);

        $this->assertFalse($a === null, "There should be a class");

        $this->assertEquals($a, $b, "Both classes should be the same");
    }

    public function testGetVarStoreTypo3() {
        // T3 Bodytext
        $this->store->setVar(TYPO3_FORM, "testformnameDoNotChange", STORE_TYPO3);
        $this->assertEquals('testformnameDoNotChange', $this->store->getVar(TYPO3_FORM, STORE_TYPO3), "System: " . TYPO3_FORM);

        // T3 Bodytext
        $this->store->setVar(TYPO3_DEBUG_SAVE, "6", STORE_TYPO3);
        $this->assertEquals('6', $this->store->getVar(TYPO3_DEBUG_SAVE, STORE_TYPO3), "System: " . TYPO3_DEBUG_SAVE);

        // T3 Bodytext
        $this->store->setVar(TYPO3_DEBUG_LOAD, "7", STORE_TYPO3);
        $this->assertEquals('7', $this->store->getVar(TYPO3_DEBUG_LOAD, STORE_TYPO3), "System: " . TYPO3_DEBUG_LOAD);

    }

    public function testGetVarStoreSystem() {
        // DBUSER in qfq.ini
        $dbuser = $this->store->getVar(SYSTEM_DBUSER, STORE_SYSTEM);
        $this->assertTrue($dbuser !== false && $dbuser !== '', "System: DBUSER found in qfq.ini");

    }

    public function testSetVarStoreSystem() {

        // Sessionname: default value
        $this->assertEquals('qfq', $this->store->getVar(SYSTEM_SESSIONNAME, STORE_SYSTEM), "System: SESSIONNAME");

        // set new Sessionname
        $this->store->setVar(SYSTEM_SESSIONNAME, "anothersessionname", STORE_SYSTEM);

        $this->assertEquals('anothersessionname', $this->store->getVar(SYSTEM_SESSIONNAME, STORE_SYSTEM), "System: SESSIONNAME");

    }


    public function testGetVarStoreClient() {

        # Violates SANITIZE class: sanitized string is always an empty string.
        # Access are cached:

        # Test: Retrieve a variable, default sanitize class
        $this->assertEquals('1234', $this->store->getVar(CLIENT_RECORD_ID, STORE_CLIENT), "FormName: default sanitize class");

        # violates default SANITIZE digit: sanitized string is always an empty string.
        $this->assertEquals('', $this->store->getVar(CLIENT_SIP, STORE_CLIENT), "sanitize class digit fails");

        // Test GET
        $this->assertEquals('1234', $this->store->getVar('key01', STORE_CLIENT), "Param: GET");

        // Test POST
        $this->assertEquals('2345', $this->store->getVar('key02', STORE_CLIENT), "Param: POST");

        // Test not found
        $this->assertFalse($this->store->getVar('keyUnknown', STORE_CLIENT), "Param: unknown");

        // Test Cache value
        $_POST['key03'] = 'lost';
        $this->store->getVar('key03', STORE_CLIENT);
        $this->assertEquals('3456', $this->store->getVar('key03', STORE_CLIENT), "Param: read from cache");

        // Test Cache not found
        $this->store->getVar('keyUnknwon2');
        $this->assertFalse($this->store->getVar('keyUnknown2', STORE_CLIENT), "Param: unknown from cache");

        // Test overwrite default sanitize class
        $this->assertEquals('', $this->store->getVar(CLIENT_FORM, STORE_CLIENT, SANITIZE_ALLOW_DIGIT), "Param: overwrite default sanitize class");

        // Test: POST higher priority than GET
        $this->assertEquals('5678', $this->store->getVar('key04', STORE_CLIENT), "Param: POST higher priority than GET");
    }


    public function testStorePriority() {
        //default prio  FSRD

        $this->store->fillStoreTableDefaultColumnType('Person');
        $this->assertEquals('male', $this->store->getVar('gender'), "Get default definition from table person.gender");

        $this->store->setVar('gender', 'female', STORE_RECORD);
        $this->assertEquals('female', $this->store->getVar('gender'), "Retrieve 'gender' from STORE_RECORD");

        $this->store->setVar('gender', 'male2', STORE_SIP);
        $this->assertEquals('male2', $this->store->getVar('gender'), "Retrieve 'gender' from STORE_SIP");

        $this->store->setVar('gender', 'female2', STORE_FORM);
        $this->assertEquals('female2', $this->store->getVar('gender', '', SANITIZE_ALLOW_ALNUMX), "Retrieve 'gender' from STORE_SIP");

    }

    public function testStoreDifferentSanitizeClass() {
        //default prio  FSRD

        $this->store->setVar('fruit', 'apple', STORE_RECORD);
        $this->assertEquals('apple', $this->store->getVar('fruit'), "Retrieve 'fruit' from STORE_RECORD");

        $this->store->setVar('color', 'green', STORE_FORM);
        $this->assertEquals(false, $this->store->getVar('color'), "Retrieve 'color' from STORE_FORM");
        $this->assertEquals('green', $this->store->getVar('color', '', SANITIZE_ALLOW_ALNUMX), "Retrieve 'color' from STORE_FORM");
    }

    public function testGetVarStore0() {
        //default prio  FSRD

        $this->assertEquals(0, $this->store->getVar('fakename', STORE_ZERO), "Retrieve anything from STORE_ZERO");
    }

    public function testUnsetStore() {
        $this->store->unsetStore(STORE_RECORD);

        $this->store->setVarArray(array(), STORE_RECORD);
        $this->assertEquals(false, $this->store->getVar('apple', STORE_RECORD), "Retrieve a value from store.");
    }

    public function testSetVarArray() {
        $this->store->unsetStore(STORE_RECORD);
        $arr = ['a' => '1', 'apple' => 'green'];
        $this->store->setVarArray($arr, STORE_RECORD);
        $this->assertEquals('1', $this->store->getVar('a', STORE_RECORD), "Retrieve a value from store 'fake'");
        $this->assertEquals('green', $this->store->getVar('apple', STORE_RECORD), "Retrieve a value from store.");
    }


    public function testSetVarArrayEmpty() {
        $this->store->unsetStore(STORE_RECORD);
        $arr = array();
        $this->store->setVarArray($arr, STORE_RECORD);
        $this->assertEquals(false, $this->store->getVar('apple', STORE_RECORD), "Retrieve a value from store.");
    }

    /**
     * @expectedException \qfq\UserException
     *
     */
    public function testGetStore() {
        $this->assertEquals(array(), $this->store->getStore('unknownstore'));
    }

}
