/**
 * @author Rafael Ostertag <rafael.ostertag@math.uzh.ch>
 */

/* global describe */
/* global it */
/* global expect */
/* global QfqNS */
/* global beforeAll */
/* global beforeEach */
/* global jasmine */

describe("Helper FunctionList", function () {
        'use strict';

        var functionList;

        beforeEach(function () {
            functionList = new QfqNS.Helper.FunctionList();
        });

        it("should call one function properly", function () {
            var spy = jasmine.createSpy('SingleFunction');

            functionList.addFunction(spy);
            functionList.call("1", 2, {});

            expect(spy).toHaveBeenCalled();
            expect(spy).toHaveBeenCalledWith("1", 2, {});

        });

        it("should call several functions properly", function () {
            var spy1 = jasmine.createSpy('Function1');
            var spy2 = jasmine.createSpy('Function2');
            var spy3 = jasmine.createSpy('Function3');

            functionList.addFunction(spy1);
            functionList.addFunction(spy2);
            functionList.addFunction(spy3);

            functionList.call("1", 2, {});

            expect(spy1).toHaveBeenCalled();
            expect(spy1).toHaveBeenCalledWith("1", 2, {});

            expect(spy2).toHaveBeenCalled();
            expect(spy2).toHaveBeenCalledWith("1", 2, {});

            expect(spy3).toHaveBeenCalled();
            expect(spy3).toHaveBeenCalledWith("1", 2, {});

        });

        it("should pass proper object as `this'", function () {
            var obj = {
                val: 0,
                f: function (a) {
                    this.val = a;
                }
            };

            var spy1 = jasmine.createSpy('Function1');

            functionList.addFunction(obj.f, obj);
            functionList.addFunction(spy1);

            functionList.call(2);

            expect(spy1).toHaveBeenCalledWith(2);
            expect(obj.val).toBe(2);

        });

        it("should not allow adding anything besides FunctionS", function () {
            expect(function () {
                functionList.add('bla');
            }).toThrowError(TypeError);
        });
    }
);