<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/30/16
 * Time: 7:59 PM
 */

namespace qfq;

require_once(__DIR__ . '/../qfq/store/Store.php');
require_once(__DIR__ . '/../qfq/Constants.php');
require_once(__DIR__ . '/../qfq/Evaluate.php');
//require_once(__DIR__ . '/../qfq/exceptions/UserException.php');
//require_once(__DIR__ . '/../qfq/exceptions/CodeException.php');
//require_once(__DIR__ . '/../qfq/exceptions/DbException.php');
//require_once(__DIR__ . '/../qfq/Evaluate.php');


class Save {

    private $formSpec = array();  // copy of the loaded form
    private $feSpecAction = array(); // copy of all formElement.class='action' of the loaded form
    private $feSpecNative = array(); // copy of all formElement.class='native' of the loaded form
    /**
     * @var null|Store
     */
    private $store = null;
    private $db = null;

    private $evaluate = null;

    /**
     * @param array $formSpec
     * @param array $feSpecAction
     * @param array $feSpecNative
     */
    public function __construct(array $formSpec, array $feSpecAction, array $feSpecNative) {
        $this->formSpec = $formSpec;
        $this->feSpecAction = $feSpecAction;
        $this->feSpecNative = $feSpecNative;
        $this->store = Store::getInstance();
        $this->db = new Database();
        $this->evaluate = new Evaluate($this->store, $this->db);
    }

    /**
     * Starts save process. On succcess, returns forwardmode/page.
     *
     * @throws CodeException
     * @throws DbException
     * @throws UserException
     */
    public function process() {

        if ($this->formSpec['multiMode'] !== 'none') {

            $parentRecords = $this->db->sql($this->formSpec['multiSql']);
            foreach ($parentRecords as $row) {
                $this->store->setVarArray($row, STORE_PARENT_RECORD, true);
                $this->elements($row['_id']);
            }
        } else {
            $this->elements($this->store->getVar(SIP_RECORD_ID, STORE_SIP . STORE_ZERO));
        }
    }

    /**
     * @param $recordId
     * @return string
     * @throws CodeException
     * @throws DbException
     * @throws UserException
     */
    public function elements($recordId) {
        $html = '';
        $newValues = array();

        $tableColumns = array_keys($this->store->getStore(STORE_TABLE_COLUMN_TYPES));
        $clientValues = $this->store->getStore(STORE_CLIENT);

        $sipValues = $this->store->getStore(STORE_SIP);

        // Iterate over all table.columns. Built an assoc array $newValues.
        foreach ($tableColumns AS $column) {
            // Never save a predefined 'id'.
            if ($column === 'id')
                continue;

            // Get related formElement.
            $formElement = $this->getFormElementByName($column);
            if ($formElement === false)
                continue;

            // Log / Debug
            $this->store->setVar(SYSTEM_FORM_ELEMENT, $formElement['name'] . ' / ' . $formElement['id'], STORE_SYSTEM);

            if ($formElement['type'] == 'hidden') {
                // Hidden elements will be transferred by SIP
                if (!isset($sipValues[$column])) {
                    throw new CodeException("Missing the hidden field '$column' in SIP.", ERROR_MISSING_HIDDEN_FIELD_IN_SIP);
                }
                $newValues[$column] = $sipValues[$column];
                continue;
            }

            $clientFieldName = HelperFormElement::buildFormElementId($column, $recordId);

            if (isset($clientValues[$clientFieldName])) {
                $newValues[$column] = $this->validateValue($formElement, $clientValues[$clientFieldName]);
            }

        }

        return ($recordId == 0) ? $this->insertRecord($this->formSpec['tableName'], $newValues) : $this->updateRecord($this->formSpec['tableName'], $newValues, $recordId);
    }

    /**
     * Get the complete FormElement for $name
     *
     * @param $name
     * @return bool|array if found the FormElement, else false.
     */
    private function getFormElementByName($name) {
        foreach ($this->feSpecNative as $formElement) {
            if ($formElement['name'] === $name)
                return $formElement;
        }
        return false;
    }

    /**
     * Check $value against given checkType/pattern. Throws an UserException if check fails.
     *
     * @param $formElement
     * @param $value
     * @return string
     * @throws CodeException
     * @throws UserException
     */
    public function validateValue($formElement, $value) {
        $pattern = '';
        $minMax = array();

        $this->store->setVar(SYSTEM_FORM_ELEMENT, $formElement['name'], STORE_SYSTEM);

        switch ($formElement['checkType']) {
            case 'min|max':
                $valueCompare = $value;

                $minMax = explode('|', $formElement['checkPattern']);
                if ($minMax[0] === '' || $minMax[1] === '') {
                    $this->store->setVar(SYSTEM_FORM_ELEMENT_MESSAGE, 'Missing definition of value for min or max.', STORE_SYSTEM);
                    throw new UserException('Missing definition of value for min or max.', ERROR_MISSING_MIN_MAX);
                }
                $errorText = "Value '$value' is smaller than min '$minMax[0]' or bigger than max '$minMax[1]'.";

                // Date/Time: convert in object to make a comparison.
                if (preg_match('date|time', $formElement['type']) === 1) {
                    $valueCompare = date_create($value);
                    $minMax[0] = date_create($minMax[0]);
                    $minMax[1] = date_create($minMax[1]);
                }

                if ($minMax[0] <= $valueCompare && $valueCompare <= $minMax[1])
                    return $value;

                $this->store->setVar(SYSTEM_FORM_ELEMENT_MESSAGE, $errorText, STORE_SYSTEM);
                throw new UserException($errorText, ERROR_MIN_MAX_VIOLATION);

                break;
            case 'pattern':
                $pattern = $formElement['checkPattern'];
                break;

            case 'number':
            case 'email':
            $arr = OnArray::inputCheckPatternArray();
            $htmlPatternAttribute = $arr[$formElement['checkType']];
                // remove 'pattern="' and closing ".
                $pattern = substr($htmlPatternAttribute, 9, strlen($htmlPatternAttribute) - 9 - 1);
                break;

            case '': // no checktype specified.
                return $value;

            default:
                throw new CodeException("Unknown checkType: " . $formElement['checkType'], ERROR_UNKNOWN_CHECKTYPE);
        }

        if (preg_match($pattern, $value) === 1)
            return $value;

        $errorText = "Value $value violates checkrule " . $formElement['checkType'] . " with pattern '$pattern'.";
        $this->store->setVar(SYSTEM_FORM_ELEMENT_MESSAGE, $errorText, STORE_SYSTEM);
        throw new UserException($errorText, ERROR_PATTERN_VIOLATION);
    }

    /**
     * Insert new record in table $this->formSpec['tableName'].
     *
     * @param array $values
     * @return string
     * @throws DbException
     */
    public function insertRecord($tableName, array $values) {

        if (count($values) === 0)
            return 0;

        $paramList = str_repeat('?, ', count($values));
        $paramList = substr($paramList, 0, strlen($paramList) - 2);
        $columnList = '`' . implode('`, `', array_keys($values)) . '`';

        $sql = 'INSERT INTO ' . $tableName . ' ( ' . $columnList . ' ) VALUES ( ' . $paramList . ' )';

        $this->db->sql($sql, ROW_REGULAR, array_values($values));

        return $this->db->getLastInsertId();
    }

    /**
     * @param array $values
     * @param $recordId
     * @return string
     * @throws DbException
     */
    public function updateRecord($tableName, array $values, $recordId) {

        if (count($values) === 0)
            return;

        if ($recordId === 0)
            throw new CodeException('RecordId=0 - this is not possible for update.', ERROR_RECORDID_0_FORBIDDEN);

        $paramList = str_repeat('?, ', count($values));
        $paramList = substr($paramList, 0, strlen($paramList) - 2);

        $sql = 'UPDATE `' . $tableName . '` SET ';

        foreach ($values as $column => $value) {
            $sql .= '`' . $column . '` = ?, ';
        }

        $sql = substr($sql, 0, strlen($sql) - 2) . ' WHERE id = ?';
        $values[] = $recordId;

        $this->db->sql($sql, ROW_REGULAR, array_values($values));
    }

}