<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2010 Glowbase GmbH <support@glowbase.com>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace qfq;

//use qfq;

require_once(__DIR__ . '/Define.php');
require_once(__DIR__ . '/Error.php');
require_once(__DIR__ . '/Log.php');


class Db {

    public $t3_typo_db_host = "";
    /**
     * @var string
     */
    private $lastUsedDB = "";
    /**
     * @var array
     */
    private $arrDB = array();
    /**
     * @var
     */
    private $t3_typo_db_username, $t3_typo_db, $t3_typo_db_password;
    /**
     * @var Log
     */
    private $log;

    // Emulate global variable: will be set much earlier in other functions. Will be shown in error messages.
    private $fr_error;


    /**
     * Constructor:
     *
     * @param    Log   fully created for logging.
     *
     */

    public function __construct($log) {
        // CR 25.4.11: require_once does not work here. No idea why
        require(PATH_typo3conf . 'localconf.php');
        $this->t3_typo_db_host = $typo_db_host;
        $this->t3_typo_db_username = $typo_db_username;
        $this->t3_typo_db = $typo_db;
        $this->t3_typo_db_password = $typo_db_password;

        $this->log = $log;
    }

    /**
     * Set Array fr_error: setter function to set most recent values, especially fr_erro['row'].
     * Will be shown in error messages.
     *
     * @param array $fr_error uid, pid, row, column_idx, full_level
     */
    public function set_fr_error(array $fr_error) {
        $this->fr_error = $fr_error;
    }

    /**
     * doQueryKeys: See doQuery
     *                Difference: fake Array for $keys
     *
     * @param string $dbAlias
     * @param string $sql
     * @param array $result
     * @param string string $expect
     * @param string $merge
     * @return bool
     * @throws CodeReportException
     * @throws SqlReportException
     * @throws SyntaxReportException
     */
    public function doQuery($dbAlias, $sql, array &$result, $expect = ROW_REGULAR, $merge = MERGE_NONE) {
        return ($this->doQueryKeys($dbAlias, $sql, $result, $fake, $expect, $merge, MYSQL_ASSOC));
    }

    /**
     * doQueryKeys: fires a show, select, insert, update or delete and collects result.
     *            insert, update and delete will produce a log entry.
     *            If: $expect==EXPECT_SQL_OR_STRING, '$sql' can be anything which won't be fired if it's not a SQL statement.
     *
     * @param string $dbAlias Name of Database to be used.
     * @param string $sql Select Query
     * @param array $result content depends on $sql.
     *                                    $sql='insert ...': mysql_last_insert_id will be returned in $result.
     *                                    $sql='update ...' or 'delete ----': mysql_affected_rows will be returned in $result.
     *                                $sql='select ...': all selected rows will be returned in $result.
     *                                $result will be formatted like specified in $merge.
     *                                    Attention: with EXPECT_1|EXPECT_0_1 '$result' is a one dimensional array, else a two dimensional array.
     * @param array $keys
     * @param string $expect
     * @param string $merge Applies different modes of merging - MERGE_NONE, MERGE_ROW, MERGE_ALL
     * @param int $arrayMode
     * @return bool                     true: all ok
     *                                  false:    a) Number of rows don't match $expect
     *                                        b) $expect==EXPECT_SQL_OR_STRING and $sql is not an SQL expression (instead it's a regular string) - this is not bad, just to indicate that there was no query.
     * @throws CodeReportException
     * @throws SqlReportException
     * @throws SyntaxReportException
     */
    public function doQueryKeys($dbAlias, $sql, array &$result, array &$keys, $expect = ROW_REGULAR, $merge = MERGE_NONE, $arrayMode = MYSQL_NUM) {
        $result = "";
        $tmp = "";
        $action = "";

        $this->selectDB($dbAlias);

        if ($this->fr_error["debug_level"] >= DEBUG_SQL) {
            // T3 function: debug()
//            debug(array('SQL' => $sql));
        }

        // Extract first parameter to check if it is a SQL statement
        $tmp = explode(" ", trim($sql), 2);
        $action = strtolower($tmp[0]);
        switch ($action) {
            case "show"  :
            case "select":
            case "insert":
            case "update":
            case "delete":
                break;  // SQL Statement: go further
            default:
                if ($expect == EXPECT_SQL_OR_STRING) {
                    $result = $sql;
                    return (false);    // nothing bad, just to indicate $sql was not a SQL statement.
                } else
                    throw new SyntaxReportException ("Unexpected SQL Statement: '$action'", "", __FILE__, __LINE__, array("DB:$dbAlias", "SQL:$sql"), $this->fr_error);
        }

        // Fire SQL statement
        if (!($res = mysql_query($sql))) {
            // Escape query if in AJAX mode
            $sql = (isset($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest') ? addslashes($sql) : $sql;
            throw new SqlReportException ("Did not get result for query.", $sql, __FILE__, __LINE__, $this->fr_error);
        }


        switch ($action) {
            case "show"  :
            case "select":
                $action = QUERY; // aggregate 'SLELECT' and 'SHOW' and ...
                $num_rows = mysql_num_rows($res);
                break;

            case "insert":
                $num_rows = mysql_affected_rows();
                $result = mysql_insert_id();
                break;
            case "update":
            case "delete":
                $num_rows = mysql_affected_rows();
                $result = $num_rows;
                break;

            default:
                throw new CodeReportException ("This error should be catched 20 lines above.", __FILE__, __LINE__);  // can't be happen, should already be detected earlier.
        }

        // Logging
        if ($action != QUERY) {
            // Not a query: write log and go home.
            $this->log->log_sql('db', '', $num_rows, $result, $sql);
            return (TRUE);
        } else {
            // Logging if localconf log_level >=D2
            $this->log->log_do('error', 'D2', '-', $sql);
        }

        // Check $expect against real result.
        switch ($expect) {
            case ROW_EXPECT_0:
                return ($num_rows == 0);
                break;
            case ROW_EXPECT_1:
                if ($num_rows != 1) return (false);
                break;
            case ROW_EXPECT_0_1:
                if ($num_rows > 1) return (false);
                break;
//            case ROW_EXPECT_GE_0:
//                break;
            case ROW_EXPECT_GE_1:
                if ($num_rows == 0) return (false);
                break;
            case EXPECT_SQL_OR_STRING:
                break;
            default:
                throw new CodeReportException ("Unknown 'expect' qualifier: $expect", __FILE__, __LINE__);
                break;
        }

        // Preparation to fetch all rows
        $tmp = "";
        $fieldCount = 0;

        // Fetch all rows:
        while ($row = mysql_fetch_array($res, $arrayMode)) {
            foreach ($row as $key => $value) {
                $row[$key] = stripslashes($value);
            }

            switch ($merge) {
                case MERGE_NONE:
                    $tmp[] = $row;
                    break;
                case MERGE_ROW:
                    $tmp[] = implode($row);
                    break;
                case MERGE_ALL:
                    $tmp .= implode($row);
                    break;
                default:
                    throw new CodeReportException ("Unknown 'merge' qualifier: $merge", __FILE__, __LINE__);
                    break;
            }
        }

        // Collect 'keys'
        if ($merge == MERGE_NONE) {
            $keys = array();
            $numberfields = mysql_num_fields($res);

            for ($i = 0; $i < $numberfields; $i++) {
                $keys[] = mysql_field_name($res, $i);
            }
        }

        // adjust Result Array: one or two dimensions.
        switch ($expect) {
            case ROW_EXPECT_0:
                break;
            case ROW_EXPECT_1:
            case ROW_EXPECT_0_1:
            case EXPECT_SQL_OR_STRING:
                $result = $tmp[0];
                break;
            default:
                $result = $tmp;
        }

        if ($merge == MERGE_ALL)
            $result = $tmp;

        mysql_free_result($res);

        return (TRUE);
    } // doQueryKeys()

    /**
     * select DB
     *
     * @param    string $dbAlias : Name of the dbname
     *
     * @return    bool        TRUE if ok, else exception.
     */
    public function selectDB($dbAlias) {

        if (!$dbAlias)
            throw new CodeReportException ("Failed: empty dbAlias", __FILE__, __LINE__);

        // If the db is still selected: do nothing.
        if ($dbAlias == $this->lastUsedDB)
            return true;

        // if the db is already open - just select it.
        if (isset($this->arrDB[$dbAlias]['link'])) {
            if (!mysql_select_db($this->arrDB[$dbAlias]['db']))
                throw new SqlReportException ("Failed: mysql_select_db($this->arrDB[$dbAlias]['db'])", "", __FILE__, __LINE__, $this->fr_error);
            return true;
        }

        $this->openDB($dbAlias);

        return true;
    } // openDB()

    /**
     * Open specified DB
     *
     * @param string $dbAlias Name of database to be opened
     * @throws SqlReportException
     */
    public function openDB($dbAlias) {
        // TYPO3 globale Variablen
// 		global $typo_db_host,$typo_db_username,$typo_db,$typo_db_password;
//		Du sollst kein global verwenden!!

        if ($dbAlias == T3) {
            $host = $this->t3_typo_db_host;
            $username = $this->t3_typo_db_username;
            $db = $this->t3_typo_db;
            $password = $this->t3_typo_db_password;
        } else {
//			require(PATH_typo3conf.'ext/formreport/ext_localconf.php');
            $host = $GLOBALS['TYPO3_CONF_VARS'][FORMREPORT][$dbAlias]['host'];
            $username = $GLOBALS['TYPO3_CONF_VARS'][FORMREPORT][$dbAlias]['username'];
            $db = $GLOBALS['TYPO3_CONF_VARS'][FORMREPORT][$dbAlias]['name'];
            $password = $GLOBALS['TYPO3_CONF_VARS'][FORMREPORT][$dbAlias]['password'];

        }

        // If 't3' is specified or the custom DB is not fully specified, take credentials from localconf.php
#		$host     = $host     ? $host 		: $typo_db_host;
#		$username = $username ? $username	: $typo_db_username;
#		$db       = $db       ? $db 		: $typo_db;
#		$password = $password ? $password 	: $typo_db_password;


        // MySQL Connect
        if (!($link = mysql_connect($host, $username, $password)))
            throw new SqlReportException ("mysql_connect($host, $username)", "", __FILE__, __LINE__, $this->fr_error);

        // Set connection charset
        if (!mysql_set_charset('utf8', $link))
            throw new SqlReportException ("mysql_set_charset('utf8', $link)", "", __FILE__, __LINE__, $this->fr_error);

        // MySQL select
        if (!mysql_select_db($db, $link))
            throw new SqlReportException ("mysql_select_db($db)", "", __FILE__, __LINE__, $this->fr_error);

        // Remember that this DB has been opened.
        $this->arrDB[$dbAlias]['link'] = $link;
        $this->arrDB[$dbAlias]['db'] = $db;

        // Remember the new DB as 'last used'
        $this->lastUsedDB = $dbAlias;

    } // selectDB()

    /**
     * closeAllDB
     *
     * @return    void
     */
    public function closeAllDB() {

        foreach ($this->arrDB as $key => $value) {
            if ($key != T3) {
                mysql_close($value);
                $arrDB[$key] = null;
            }
        }

    } // closeAllDB()
}
