<?PHP
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2010 Glowbase GmbH <support@glowbase.com>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace qfq;

//use qfq;

require_once(__DIR__ . '/Define.php');
require_once(__DIR__ . '/Utils.php');
require_once(__DIR__ . '/../store/Store.php');
require_once(__DIR__ . '/../store/Sip.php');

/*
 * u:url
 * m:mailto
 * p:page
 * 
 * t:text
 * a:AltText
 * o:ToolTip
 * 
 * r:render   
 * g:target
 * c:class  [n|i|e]:<class>
 * q:question  <text>
 * e:encryption 0|1
 * 
 * E:edit
 * N:new
 * D:delete
 * H:Help
 * I:information
 * M:Magnifier
 * B:bullet
 * P:picture       [file]
 * C:checkbox    [name]
 * R:right 
 * h:hash  
 *
 * A:    A:[u:p:m]
 * G:    G:[N|..]
 */

class Link {

    private $mailto = "", $url = "", $image = "", $text = "", $altText = "", $imageTitle = "", $question = "", $target = "", $additionalUrlParams = "";
    private $toolTip = array(), $modeRender = 0, $picturePosition = "l", $modeHash = false, $encryption = 1;
    private $linkClass = "", $defaultLinkClass = "";
    private $linkClassSelector = array("i" => "internal", "e" => "external");
    private $renderControl, $link = "";

    /**
     * @var Sip
     */
    private $sip = null;

    /**
     * @var Store
     */
    private $store = null;

    // Simulate global variable: will be set much earlier in other functions. Will be shown in error messages.
    private $fr_error;

    private $exceptionBody = "";
    /**
     * @var Utils
     */
    private $utils;
    /**
     * @var string
     */
    private $pageAlias = '';

    private $cssLinkClassInternal = '';
    private $cssLinkClassExternal = '';

    /**
     * __construct
     *
     * @param string $fullLevel Recent processed level, f.e.: '10.20.sql'
     * @param string $sql Recent processed query, f.e.: SELECT p.name FROM person AS p
     * @param string $columnValue Definition of link
     * @param int $count
     * @param int $columnIndex
     * @param string $fr_error
     * @param string $dbAlias
     */
    public function __construct($fullLevel, $sql, $columnValue, $count, $columnIndex, $fr_error, $dbAlias = '', $sip) {
        $this->exceptionBody = "Level: " . $fullLevel . ".sql = " . $sql . "\nRow: " . $count . " , Column: " . $columnIndex . " , Bad link: " . $columnValue . "\n";
        $this->fr_error = $fr_error;
        $this->dbAlias = $dbAlias;
        $this->sip = $sip;
        $this->store = Store::getInstance();

        $this->cssLinkClassInternal = $this->store->getVar(SYSTEM_CSS_LINK_CLASS_INTERNAL, STORE_SYSTEM);
        $this->cssLinkClassExternal = $this->store->getVar(SYSTEM_CSS_LINK_CLASS_EXTERNAL, STORE_SYSTEM);
        $this->initRenderControl();

    }

    /**
     * Initializes RenderControl Array
     *
     * @param    void
     * @return    void
     */
    private function initRenderControl() {

        $this->utils = new Utils();

        /*
         * mode:
         * 0: no output
         * 1: text
         * 2: url
         * 3: <a href=url>url</a>
         * 4: <a href=url>Text</a>
         *
         *  r=render mode, u=url, t:text and/or image.
         *
         *                  [r][u][t] = mode
         */

        $this->renderControl[0][0][0] = 0;
        $this->renderControl[0][0][1] = 0;
        $this->renderControl[0][1][0] = 3;
        $this->renderControl[0][1][1] = 4;

        $this->renderControl[1][0][0] = 0;
        $this->renderControl[1][0][1] = 1;
        $this->renderControl[1][1][0] = 3;
        $this->renderControl[1][1][1] = 4;

        $this->renderControl[2][0][0] = 0;
        $this->renderControl[2][0][1] = 0;
        $this->renderControl[2][1][0] = 0;
        $this->renderControl[2][1][1] = 4;

        $this->renderControl[3][0][0] = 0;
        $this->renderControl[3][0][1] = 1;
        $this->renderControl[3][1][0] = 2;
        $this->renderControl[3][1][1] = 1;

        $this->renderControl[4][0][0] = 0;
        $this->renderControl[4][0][1] = 1;
        $this->renderControl[4][1][0] = 2;
        $this->renderControl[4][1][1] = 2;

        $this->renderControl[5][0][0] = 0;
        $this->renderControl[5][0][1] = 0;
        $this->renderControl[5][1][0] = 0;
        $this->renderControl[5][1][1] = 0;
    }

    /**
     * Build the whole link
     *
     * @param    string $str :  Qualifier with params. 'report'-syntax. F.e.:  A:u:www.example.com|G:P:home.gif|t:Home"
     * @param    string $count
     * @param    string &$hash hash used by the link, and maybe created for the link if count=1
     *
     * @return    string      The complete Link
     */
    public function renderLink($str, $count, &$hash) {

        // str="u:http://www.example.com|c:i|t:delete"
        $parm = explode("|", $str);

        // Parse all parameter, fill variables
        foreach ($parm as $item) {
            $arr = explode(":", $item, 2);
            $this->parseItem($arr[0], $arr[1]);
        }

        // if there is no url or mailto definition: ~global.pageId
        if (!$this->url && !$this->mailto) $this->url = "?" . $this->store->getVar(TYPO3_PAGE_ID, STORE_TYPO3);

        $this->doCssClass();

        $htmlUrl = $this->doUrl($count, $hash);
        $htmlImage = $this->doImage();

        // One space distance
        if ($this->text) $distance = " ";

        // Compose Image & Text
        $distance = ' ';
        $this->text = ($this->picturePosition == "l") ? $htmlImage . $distance . $this->text : $this->text . $distance . $htmlImage;

        // ToolTip
        $extraSpan = array();
        if ($this->toolTip) {
            $extraSpan[0] = "<span " . $this->toolTip[0] . ">" . $this->toolTip[1];
            $extraSpan[1] = "</span>";
        }

        // Create 'fake' modes for encrypted 'mailto'
        $prefix = "";
        if ($this->mailto) {
            $prefix = "1";
            $this->url = "dummy";
        }
        // Create 'fake' mode for ajax delete
        if ($this->delete) {
            $prefix = "2";
        }
        // Compose URL
        // get Render Mode via Array renderControl
        $mode = $prefix . $this->renderControl[$this->modeRender][$this->url ? 1 : 0][$this->text ? 1 : 0];
        // 0-4 URL, plain email
        // 10-14 encrypted email
        switch ($mode) {
            // 0: No Output
            case '0':
            case '10':
            case '20':
                $this->link = "";
                break;

            // 1: 'text'
            case '1':
                $this->link = $extraSpan[0] . $this->text . $extraSpan[1];
                break;
            case '11':
                $this->link = $extraSpan[0] . $this->encryptMailtoJS($this->mailto, $this->text, $this->linkClass, false) . $extraSpan[1];
                break;

            // 2: 'url'
            case '2':
                $this->link = $extraSpan[0] . $this->url . $extraSpan[1];
                break;
            case '12':
                $this->link = $extraSpan[0] . $this->encryptMailtoJS($this->mailto, $this->text, $this->linkClass, false) . $extraSpan[1];
                break;

            // 3: <a href=url>url</a>
            case '3':
                $this->link = $htmlUrl . $this->url . '</a>' . $this->toolTip[1];
                break;
            case '13':
                $this->link = $this->encryptMailtoJS($this->mailto, $this->mailto, $this->linkClass, TRUE);
                break;

            // 4: <a href=url>Text</a>
            case '4':
                $this->link = $htmlUrl . $this->text . '</a>' . $this->toolTip[1];
                break;
            case '14':
                $this->link = $this->encryptMailtoJS($this->mailto, $this->text, $this->linkClass, TRUE);
                break;
            case '21':
            case '22':
            case '23':
            case '24':
                $this->link = "<a href=\"javascript: void(0);\" onClick=\"var del = new FR.Delete({recordId:'" . $count . "',hash:'" . $hash . "',forward:'" . $this->pageAlias . "'});\" " . $this->linkClass . ">" . $this->text . "</a>";
        }

        return $this->link;

    }

    /**
     * Parse Item of link string, fill class global variables.
     *
     * @param    string $key :    F.e.: 'u'
     * @param    string $value :  F.e.: 'http://www.nzz.ch'
     * @throws SyntaxReportException
     */
    private function parseItem($key, $value) {

        switch ($key) {
            // URL
            case "u":
                if ($this->url || $this->mailto) throw new SyntaxReportException ("Multiple URL, PAGE or MAILTO defined : '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->url = $value;
                $this->defaultLinkClass = $this->cssLinkClassExternal;
                break;
            case "m":
                if ($this->url || $this->mailto) throw new SyntaxReportException ("Multiple URL, PAGE or MAILTO defined : '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->mailto = $value;
                $this->defaultLinkClass = $this->cssLinkClassExternal;
                break;
            case "p":
                if ($this->url || $this->mailto) throw new SyntaxReportException ("Multiple URL, PAGE or MAILTO defined : '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->pageAlias = $value ? $value : $this->store->getVar(TYPO3_PAGE_ID, STORE_TYPO3); // If no pageid|pagealias is defined, take current page
//                $this->url = "?" . $this->pageAlias;
                $this->defaultLinkClass = $this->cssLinkClassInternal;
                break;
            // Text
            case "t":
                $this->text = $value;
                break;
            case "a":
                $this->altText = $value;
                break;
            case "o":
                $this->toolTip = $this->utils->createToolTip($value);
                break;

            // Image
            case "P":
                if ($this->image) throw new SyntaxReportException ("Multiple images defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->image = $value;
                $this->AltText = "Grafic: " . $value;
                $this->imageTitle = $value;
                break;
            case "B":
                if ($this->image) throw new SyntaxReportException ("Multiple images defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->image = PATH_ICONS . "bullet-" . ($value ? $value : "green") . '.gif';
                $this->defaultAltText = "Bullet: " . $value;
                $this->imageTitle = $value;
                break;
            case "C":
                if ($this->image) throw new SyntaxReportException ("Multiple images defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->image = PATH_ICONS . "checked-" . ($value ? $value : "green") . '.gif';
                $this->defaultAltText = "Checked: " . $value;
                $this->imageTitle = $value;
                break;
            case "D":
                if ($this->image) throw new SyntaxReportException ("Multiple images defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->image = PATH_ICONS . 'delete.gif';
                $this->defaultAltText = "Delete";
                $this->imageTitle = "Delete";
                $this->delete = true;
                // Include Extjs library
                $this->utils->loadJSlib($this->fr_error);
                break;
            case "E":
                if ($this->image) throw new SyntaxReportException ("Multiple images defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->image = PATH_ICONS . 'edit.gif';
                $this->defaultAltText = "Edit";
                $this->imageTitle = "Edit";
                break;
            case "H":
                if ($this->image) throw new SyntaxReportException ("Multiple images defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->image = PATH_ICONS . 'help.gif';
                $this->defaultAltText = "Help";
                $this->imageTitle = "Help";
                break;
            case "I":
                if ($this->image) throw new SyntaxReportException ("Multiple images defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->image = PATH_ICONS . 'info.gif';
                $this->defaultAltText = "Information";
                $this->imageTitle = "Information";
                break;
            case "N":
                if ($this->image) throw new SyntaxReportException ("Multiple images defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->image = PATH_ICONS . 'new.gif';
                $this->defaultAltText = "New";
                $this->imageTitle = "New";
                break;
            case "S":
                if ($this->image) throw new SyntaxReportException ("Multiple images defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->image = PATH_ICONS . 'show.gif';
                $this->defaultAltText = "Details";
                $this->imageTitle = "Details";
                break;

            // Misc
            case "r":
                if ($this->modeRender) throw new SyntaxReportException ("Multiple render modes defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->modeRender = $value;
                break;
            case "g":
                if ($this->target) throw new SyntaxReportException ("Multiple 'target' defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->target = $value;
                break;
            case "c":
                if ($this->linkClass) throw new SyntaxReportException ("Multiple linkClass defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->linkClass = $value;
                break;
            case "q":
                if ($this->question) throw new SyntaxReportException ("Multiple question defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->question = $value ? $value : "Please confirm";
                break;
            case "e":
                if ($this->encryption != 1) throw new SyntaxReportException ("Multiple encryption defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->encryption = $value ? $value : "1";
                break;
            case "h":
                if ($this->modeHash) throw new SyntaxReportException ("Multiple modeHash defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->modeHash = TRUE;
                break;
            case "U":    //if($this->additionalUrlParams) throw new syntaxException ( "Multiple additionalUrlParams defined: '$key'","",__FILE__,__LINE__, "Column: " . $fthis->r_error["columnIndex"], $this->fr_error);
                $this->additionalUrlParams = $value;
                break;
            case "R":
                if ($this->picturePosition != "l") throw new SyntaxReportException ("Multiple picturePosition defined: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
                $this->picturePosition = "r";
                break;

            // A,G
            case "G":
            case "A":
                $arr = explode(":", $value, 2);
                $this->parseItem($arr[0], $arr[1]);
                break;
            case "" :
                break;
            default:
                throw new SyntaxReportException ("Unknown link qualifier: '$key'", "", __FILE__, __LINE__, "Column: " . $this->fr_error["columnIndex"], $this->fr_error);

        }

    }

    /**
     * Parse CSS Class Settings
     *
     * @param    void
     *
     * @return    void
     */
    private function doCssClass() {

        // class: i|e|n|<custom name>
        if ($this->linkClass != 'n') {
            if (!$this->linkClass) $this->linkClass = $this->defaultLinkClass;

            if ($this->linkClassSelector[$this->linkClass])
                $this->linkClass = $this->linkClassSelector[$this->linkClass];

            $this->linkClass = ' class="' . $this->linkClass . '"';
        } else {
            $this->linkClass = "";
        }
    }

    /**
     * Builds complete HTML anchor.
     * - <a href="mailto:info@example.com:>info@example.com</a>
     * - <a href="http://example.com:>http://example.com</a>
     * - If $this->modeHash is set, create a
     *
     * @return string
     * @throws SyntaxReportException
     */
    private function doUrl() {
        // build URL
        $htmlUrl = "";

        if ($this->url) {

            if ($this->modeHash) {

                $this->url = $this->sip->queryStringToSip('id=' . $this->pageAlias . '&' . $this->additionalUrlParams);

            } else {

                if ($this->additionalUrlParams) {
                    // Append '&' or '?' depending if there is already a '?'
                    $this->url .= (strpos($this->url, '?') >= 0) ? '&' : '?';
                    // Append all additional params.
                    $this->url .= $this->additionalUrlParams;
                }
            }
        }

        // If there is no encryption: handle the mailto as an ordinary URL
        if ($this->mailto) {

            if ($this->encryption == 0) {
                $this->url = "mailto:" . $this->mailto;
                $this->mailto = "";
            } else {
                throw new SyntaxReportException ("Oops, sorry: encrypted mail not implemented ...", "", __FILE__, __LINE__,
                    "Column: " . $this->fr_error["columnIndex"], $this->fr_error);
            }
        }

        // target
        if ($this->target) $this->target = ' target="' . $this->target . '"';

        // question
        if ($this->question) $this->question = ' onclick="return confirm(\'' . $this->question . '\')"';

        $htmlUrl = '<a href="' . $this->url . '"' . $this->linkClass . $this->target . $this->question . $this->toolTip[0] . '>';

        return ($htmlUrl);
    }

    /**
     * Create Image HTML Tag
     * @return string $htmlImage
     * @internal param $void
     *
     */
    private function doImage() {
        $htmlImage = "";

        // Build Image
        if ($this->image) {

            if ($this->altText) $this->altText = ' alttext="' . $this->altText . '"';

            if ($this->image) $this->image = ' src="' . $this->image . '"';

            if ($this->imageTitle) $this->imageTitle = ' title="' . $this->imageTitle . '"';

            $htmlImage = '<img' . $this->image . $this->altText . $this->imageTitle . ' />';
        }

        return ($htmlImage);
    }

    /**
     *  Encrypt the mailto address via JS.
     *  Email address protected against email crawler (as long as they don't interpret JS).
     *
     *    <script language=javascript><!--
     *    var mm1 = name
     *    var mm2 = @domain.
     *    var ... = tld
     *    document.write("<a href=" + "mail" + "to:" + mm1 + mm2 + ... + ">" + name + "</a>")
     *    document.write("<a href=" + "mail" + "to:" + mm1 + mm2 + ... + ">" + @domain. + "</a>")
     *    document.write("<a href=" + "mail" + "to:" + mm1 + mm2 + ... + ">" + tld + "</a>")
     *    //--></script>';
     *
     * @param    string $mailto Email address to encrypt.
     * @param    string $text Text to wrap <a href..>, </a> around.
     * @param    string $class Optional class definition
     * @param    bool $href TRUE: create a '<a>',   false: just encrypt or show the email, no link.
     *
     * @return    string The complete Link
     */
    private function encryptMailtoJS($mailto, $text, $class = "", $href = TRUE) {

        // Prepare CSS Definition
        if ($class) $class = str_replace('"', "'", $class);

        // Split $mailto
        $tmp = $this->splitAndAddDelimter($mailto, "@");
        $arr = array_merge($this->splitAndAddDelimter($tmp[0], "."), $this->splitAndAddDelimter($tmp[1], "."));

        $tt = "<script language=javascript><!--" . chr(10);
        $ii = 0;
        if ($href) {
            $dw = 'document.write("<a href=" + "mail" + "to:"';
            foreach ($arr as $value) {
                // Setup JS Variables
                $tt .= 'var mm' . $ii . '= "' . $value . '"' . chr(10);
                // Compose $dw (documentwrite statement)
                $dw .= ' + mm' . $ii++;
            }
            $dw .= ' + "' . $class . str_replace('"', '\\"', $this->toolTip[0]) . '>"';
            $closeDw = '"</a>")';
        } else {
            $dw = 'document.write(';
            $closeDw = ')';
        }

        // Wrap mailto around text
        if ($mailto == $text) {
            // Text is an email: wrap every single part of mailto.
            $ii = 0;
            foreach ($arr as $value) {
                $tt .= $dw . " + mm" . $ii++ . ' + ' . $closeDw . chr(10);
            }
        } else {
            // Single wrap text
            $tt .= $dw . ' + "' . $text . '" + ' . $closeDw . chr(10);
        }

        $tt .= '//--></script>';
        if ($href) $tt .= $this->toolTip[1];

        return ($tt);
    }

    /**
     * Split a string around the $delimiter.
     *
     * Append the delimiter to each part except the last one.
     *
     * @param $mailto
     * @param $delimiter
     * @return array
     */
    private function splitAndAddDelimter($mailto, $delimiter) {
        $value = '';

        $arr = explode($delimiter, $mailto);            // split string

        foreach ($arr as $key => $value) {
            $arr[$key] = $value . $delimiter;
        }

        if (isset($key))
            $arr[$key] = $value;

        return ($arr);
    }
}