<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 1/15/16
 * Time: 8:24 AM
 */

namespace qfq;

use qfq\Store;
use qfq;

require_once(__DIR__ . '/AbstractDatabaseTest.php');
require_once(__DIR__ . '/../../qfq/Evaluate.php');
require_once(__DIR__ . '/../../qfq/Database.php');
require_once(__DIR__ . '/../../qfq/store/Store.php');

class EvaluateTest extends \AbstractDatabaseTest {

    public function testVars() {
        $eval = new \qfq\Evaluate($this->store, $this->db);

        // no variable
        $this->assertEquals('nothing', $eval->parse('nothing'));
        $this->assertEquals('TestFormName', $eval->parse('{{form:T}}'));

        //  pure digit variable
        $this->store->setVar('a', '1', 'C');
        $this->assertEquals('1', $eval->parse('{{a:C}}'));

        // variable contains variable
        $this->store->setVar('a', '{{b:C}}', 'C');
        $this->store->setVar('b', '1234', 'C');
        $this->assertEquals('1234', $eval->parse('{{a:C:all}}'));

        // multiple vaules, Recursive
        $this->store->setVar('m1', 'M1=[{{m2:C:all}}]', 'C');
        $this->store->setVar('m2', 'M2=[{{m3:C:all}}]', 'C');
        $this->store->setVar('m3', 'Some really nice text', 'C');
        $this->assertEquals('These are the variables > m1:M1=[M2=[Some really nice text]], m2:M2=[Some really nice text], m3:Some really nice text - end outer line',
            $eval->parse('These are the variables > m1:{{m1:C:all}}, m2:{{m2:C:all}}, m3:{{m3:C:all}} - end outer line'));
    }

    public function testDb() {
        $eval = new \qfq\Evaluate($this->store, $this->db);

        // database: lower case
        $this->assertEquals('1DoeJohnmalec2SmithJanefemalea,c', $eval->parse('{{select * from Person where id < 3 order by id}}'));

        // database: upper case
        $this->assertEquals('1DoeJohnmalec2SmithJanefemalea,c', $eval->parse('{{SELECT * FROM Person WHERE id < 3 ORDER BY id}}'));


        $this->store->setVar('sql', 'SELECT * FROM Person WHERE id < 3 ORDER BY id', 'C');
        $this->assertEquals('SELECT * FROM Person WHERE id < 3 ORDER BY id', $eval->parse('{{sql:C:all}}'));
        $this->assertEquals('1DoeJohnmalec2SmithJanefemalea,c', $eval->parse('{{{{sql:C:all}}}}'));

        $expected = [
            [
                'id' => '1',
                'name' => 'Doe'
            ],
            [
                'id' => '2',
                'name' => 'Smith'
            ],
        ];
        $this->assertEquals($expected, $eval->parse('{{!SELECT id, name FROM Person WHERE id < 3 ORDER BY id}}'));

        // INSERT: Use eval() to write a record
        $eval->parse('{{INSERT INTO Person (name, firstname) VALUES (\'Sinatra\', \'Frank\')}}');
        $this->assertEquals('Frank Sinatra', $eval->parse('{{SELECT firstname, " ", name FROM Person WHERE name="Sinatra" AND firstname="Frank"}}'));

        // INSERT & DELETE: Use eval() to write a record
        $eval->parse('{{INSERT INTO Person (name, firstname) VALUES (\'Sinatra\', \'Frank\')}}');
        $eval->parse('{{DELETE FROM Person WHERE name="Sinatra" AND firstname="Frank" LIMIT 1}}');
        $this->assertEquals('1', $eval->parse('{{SELECT count(id) FROM Person WHERE name="Sinatra" AND firstname="Frank"}}'));

        // UPDATE
        $eval->parse('{{UPDATE Person set name=\'Zappa\' WHERE name="Sinatra" AND firstname="Frank" LIMIT 1}}');
        $this->assertEquals('1', $eval->parse('{{SELECT count(id) FROM Person WHERE name="Zappa" AND firstname="Frank"}}'));

        // SHOW tables
        $expected = "idbigint(20)NOPRIauto_incrementnamevarchar(128)YESfirstnamevarchar(128)YESgenderenum('','male','female')NOmalegroupsset('','a','b','c')NO";

        $this->assertEquals($expected, $eval->parse('{{SHOW COLUMNS FROM Person}}'));


    }

    public function testDbVars() {
        $eval = new \qfq\Evaluate($this->store, $this->db);

        $eval->parse('{{INSERT INTO Person (name, firstname) VALUES (\'Holiday\', \'Billie\')}}');
        $this->store->setVar('a', '{{b:C:all}}', 'C');
        $this->store->setVar('b', 'Holiday', 'C');
        $this->assertEquals('Holiday', $eval->parse('{{SELECT name FROM Person WHERE name=\'{{a:C:all}}\'}}'));

    }

    public function testParseArray() {
        $eval = new \qfq\Evaluate($this->store, $this->db);

        $data = [
            'formName' => 'MyTestForm',
            'title' => 'Person: {{SELECT firstname, " ", name FROM Person LIMIT 1}} / Employee'
        ];
        $expected = [
            'formName' => 'MyTestForm',
            'title' => 'Person: John Doe / Employee'
        ];

        $this->assertEquals($expected, $eval->parseArray($data));


    }

    public function testParseArrayOfArray() {
        $eval = new \qfq\Evaluate($this->store, $this->db);
        $data = [
            [
                'formName' => 'MyTestForm',
                'title' => 'Person: {{SELECT firstname, " ", name FROM Person LIMIT 1}} / Employee'
            ],
            [
                'order' => '100',
                'class' => 'Person: {{SELECT firstname, " ", name FROM Person LIMIT 1}} / Employee'
            ]
        ];
        $expected = [
            [
                'formName' => 'MyTestForm',
                'title' => 'Person: John Doe / Employee'
            ],
            [
                'order' => '100',
                'class' => 'Person: John Doe / Employee'
            ]
        ];

        $this->assertEquals($expected, $eval->parseArray($data));
    }

    protected function setUp() {

        $this->store = Store::getInstance('form=TestFormName', true);
        parent::setUp();

        $this->store->setVar('form', 'TestFormName', STORE_TYPO3);
        $this->executeSQLFile(__DIR__ . '/fixtures/Generic.sql', true);

    }
}
